.progress-bar{
    width: 100%;
    height: 1.8rem;
    border-radius: 10px;
    position: relative;
    background-color: #C3DCDC;
}

.progress-bar[value]{
    background-color: #00824D;
}

#progress-bar-label{
    position: absolute;
    top: 145px;
    left: 25%;
    z-index: 1;
    color: white;
    font-weight: 502;
}

progress::-webkit-progress-bar {
    background-color: #C3DCDC;
}

progress::-webkit-progress-value {
    background-color: #00824D;
}

progress[value]::-moz-progress-bar{
    background-color: #00824D;
}
