.workspace-page-wrapper{
    display: flex;
    flex-wrap: wrap;
    flex-direction: row;
    padding: 0;
}

.workspace-header{
    width: 100%;
    display:flex;
    flex-wrap: wrap;
    justify-content: space-between;
    align-items: center;
    margin-bottom: 1rem;
}

.ssb-title.workspace-title{
    margin-top: 1rem;
    width: 100%;
}

.workspace-content{
    width: 100%;
    display: flex;
    flex-direction: column;
    gap: 1rem;
}

.ssb-accordion.variable-accordion > .accordion-body {
    padding: 1rem;
}

.edit-section{
    display: flex;
    flex-direction: column;
    gap: 1rem;
    width: 100%;
    padding: 1rem 0;
}

.edit-section-form{
    display: flex;
    flex-wrap: wrap;
    gap: 2rem;
    padding-top: 1rem;
    align-items: flex-start;
}

.ssb-title.edit-section-title{
    border-bottom: 2px solid #62919A;
    margin-bottom: 1rem;
    padding-bottom: 1rem;

}
